package tnviet.proj.jna.textgroup;


import tnviet.proj.jna.utilities.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;


/*
**  Group multiple MaskTextFields and JLabels to act like
**  a single component.
**
**  1)  A MaskTextField is a customized JTextField.
**  2)	The focus manager will only set focus on the first
**		text field added to the group.
**  3   The up/down/right/left arrow keys can be used to
**		move within the group.
**  4)	The getText() method returns the text of all
**		components added to the group.
**  5)  The getData() method returns the text of the
**		text fields added to the group.
*/
public class TextGroup extends JPanel implements KeyListener
{
	public final static int TAB_NEXT = 1;
	public final static int TAB_PREVIOUS = -1;
	public final static Border DEFAULT_BORDER = UIManager.getBorder( "TextField.border" );
	public final static Color DEFAULT_BACKGROUND = UIManager.getColor( "TextField.background" );
	public final static Font DEFAULT_FONT = new Font("monospaced", Font.PLAIN, 12);
	public final static Color DEFAULT_MASK_COLOR = Color.gray;
	public final static boolean DEFAULT_DISPLAY_PARTIAL_MASK = true;

	private static Border labelBorder = new EmptyBorder(0, 5, 0, 5);

	private String delimiters;
	private Color maskColor = DEFAULT_MASK_COLOR;
	private boolean displayPartialMask = DEFAULT_DISPLAY_PARTIAL_MASK;
	private MaskTextField firstField;
	private List maskTextFields;

	private StringBuffer text = new StringBuffer(16);

	/*
	**  Convenience constructors
	*/
	public TextGroup(String mask, String delimiters)
	{
		this(mask, delimiters, null, null);
	}

	public TextGroup(String mask, String delimiters, FieldValidator validator)
	{
		this(mask, delimiters, validator, null);
	}

	public TextGroup(String mask, String delimiters, FieldValidator[] validators)
	{
		this(mask, delimiters, null, validators);
	}

	/*
	**  Complete constructer
	*/
	public TextGroup(String mask, String delimiters, FieldValidator validator, FieldValidator[] validators)
	{
		this.delimiters = delimiters;
		setLayout( new FlowLayout(FlowLayout.CENTER, 0, 0 ) );
		setBorder( DEFAULT_BORDER );
		setBackground( DEFAULT_BACKGROUND );
		setFont( DEFAULT_FONT );
		maskTextFields = new ArrayList();

		//  Parse the mask into mask text fields and labels

		StringTokenizer st = new StringTokenizer(mask, delimiters, true);
		int tokenCount = 0;

		while ( st.hasMoreTokens() )
		{
			String token = st.nextToken();

			if (delimiters.indexOf( token ) == -1)
			{
				FieldValidator fv = getNextValidator( tokenCount, validator, validators );
				addTextField( token, fv );
				tokenCount++;
			}
			else
			{
				addLabel( token );
			}
		}
	}

	/*
	**  When a single FieldValidator is available it is used for all text fields.
	**  When multiple FieldValidators are available use validator corresponding to current token.
	*/
	private FieldValidator getNextValidator(int tokenCount, FieldValidator validator, FieldValidator[] validators)
	{
		if (validator != null)
			return validator;

		if (validators != null && tokenCount < validators.length)
			return validators[tokenCount];
		else
			return null;
	}

	/*
	**  Create a mask text field and add it to the panel
	*/
	public MaskTextField addTextField(String mask, FieldValidator validator)
	{
		MaskTextField textField = new MaskTextField(mask.length(), mask);
		//  uncomment the following line when using JDK1.3
//		textField.setFocusTraversable( false );
		textField.setFont( null );
		textField.setBorder( null );
		textField.setOpaque( false );
		textField.setForeground( null );
		textField.setParent( this );
		textField.setValidator( validator );
		textField.addKeyListener( this );

		//  First mask field created can accept focus from the Focus Manager

		if (maskTextFields.size() == 0)
		{
			textField.setFocusTraversable( true );
		}

		maskTextFields.add( textField );

		super.add( textField );
		return textField;
	}

	/*
	**  Create a label and add it to the panel
	*/
	public JLabel addLabel(String delimiter)
	{
		JLabel label = new JLabel( delimiter );
		super.add( label );

		label.setBorder( labelBorder );
		label.setFont( null );

		return label;
	}

	/*
	**
	*/
	public void setText(String delimitedData)
	{
		//  Parse out the data from the delimited string

		StringTokenizer st = new StringTokenizer(delimitedData, delimiters, true);
		int tokenCount = 0;

		while ( st.hasMoreTokens() )
		{
			String token = st.nextToken();

			if (delimiters.indexOf( token ) == -1)
			{
				if (tokenCount < maskTextFields.size())
				{
					setText( tokenCount, token );
				}

				tokenCount++;
			}
		}
	}

	public void setText(int position, String text)
	{
		if (position < maskTextFields.size())
		{
			((MaskTextField)maskTextFields.get( position )).setText( text );
		}
	}

	/*
	**  Return the text with delimiters
	*/
	public String getText()
	{
		return getComponentText( true );
	}

	public String getText(int position)
	{
		if (position < maskTextFields.size())
		{
			return ((MaskTextField)maskTextFields.get( position )).getText();
		}
		else
			return "";
	}

	/*
	**  Return the text without delimiters
	*/
	public String getData()
	{
		return getComponentText( false );
	}

	/*
	**  Methods to control the mask text color
	*/
	public void setMaskColor(Color color)
	{
		maskColor = color;
	}

	public Color getMaskColor()
	{
		return maskColor;
	}

	/*
	**  Control how the background mask is rendered
	**  true: as characters are entered a partial mask is displayed
	**  false: once the first character is entered no mask is displayed
	*/
	public void setDisplayPartialMask(boolean value)
	{
		displayPartialMask = value;
	}

	public boolean isDisplayPartialMask()
	{
		return displayPartialMask;
	}

	/*
	**  Append the text from the individual components
	**  to form the text for the group component.
	*/
	private String getComponentText(boolean delimiterRequired)
	{
		text.setLength( 0 );
		Component[] components = getComponents();

		for (int i = 0; i < components.length; i++)
		{
			if (components[i] instanceof JTextField)
			{
				JTextField textField = (JTextField)components[i];
				text.append( textField.getText() );
			}

			if (delimiterRequired && components[i] instanceof JLabel)
			{
				JLabel label = (JLabel)components[i];
				text.append( label.getText() );
			}
		}

		return text.toString();
	}

	/*
	**  Set focus on the next/previous text field
	*/
	public void tab(JTextField textField, int direction)
	{
		Component[] components = getComponents();

		//  find the text field that currently has focus

		int i = 0;

		while ( components[i] != textField )
			i++;

		//  find the next/previous text field on the panel

		do
		{
			i = (i + direction + components.length) % components.length;
		}
		while ( ! (components[i] instanceof JTextField) );

		//  set focus on the text field

		components[i].requestFocus();
	}

	//  Implement KeyListener
	//  Override the default behaviour of certain keys

	public void keyPressed(KeyEvent e)
	{
		JTextField textField = (JTextField)e.getSource();

		switch (e.getKeyCode())
		{
			//  tab to next text field

			case KeyEvent.VK_UP:
				tab( textField, TAB_NEXT );
				break;

			//  tab to previous text field

			case KeyEvent.VK_DOWN:
				tab( textField, TAB_PREVIOUS );
				break;

			//  move to next character or tab to next field

			case KeyEvent.VK_RIGHT:
				if (textField.getCaretPosition() == textField.getDocument().getLength())
				{
					tab( textField, TAB_NEXT );
					e.consume();
				}
				break;

			//  move to previous character or tab to previous field

			case KeyEvent.VK_LEFT:
				if (textField.getCaretPosition() == 0)
				{
					tab( textField, TAB_PREVIOUS );
					e.consume();
				}
				break;
		}
	}

	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}

	/*
	**  Test the class
	*/
	public static void main(String[] args)
	{
		JFrame frame = new JFrame();
		frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		JPanel panel = new JPanel( new GridLayout(0,2) );
		frame.setContentPane( panel );

		//  add date with no validation
		final TextGroup date1 = new TextGroup("YYYY/MM/DD", "/");
		date1.setBackground( Color.yellow );
		date1.setForeground( Color.red );
		panel.add( new JLabel("Date - not validated:") );
		panel.add( date1 );

		JButton textButton = new JButton( "Validate Date" );
		textButton.addActionListener( new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				System.out.println( date1.getText(0) ); // get text from first field
				System.out.println( date1.getData() );  // get text, excluding delimiters
				String text = date1.getText(); // get text, including delimeters
				System.out.print( text );
				try
				{
					SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
					df.setLenient( false );
					df.parse(text);
					System.out.println(" is valid");
				}
				catch(Exception e2)
				{
					System.out.println(" is invalid");
				}
			}
		});
		panel.add( new JLabel("") );
		panel.add( textButton );

		//  add date with numeric validation
		FieldValidator[] validators = new FieldValidator[3];
		validators[0] = new IntegerValidator(1900, 2010);
		validators[1] = new IntegerValidator(1, 12, 2);
		validators[2] = new IntegerValidator(1, 31, 2);
		TextGroup date2 = new TextGroup("YYYY/MM/DD", "/", validators);
		date2.setDisplayPartialMask( false );
		date2.setMaskColor( Color.red );
		panel.add( new JLabel("Date - validated:") );
		panel.add( date2 );

		//  add phone number validate for integer characters
		FieldValidator numeric = new IntegerValidator();
		TextGroup phone = new TextGroup("(###)___-____", "()-", numeric);
		panel.add( new JLabel("Phone Number:") );
		panel.add( phone );

		//  add IP address
		IntegerValidator ipv = new IntegerValidator(1, 255);
		TextGroup ip = new TextGroup("   .   .   .   ", ".", ipv);
		ip.setText("123.00.42.03");
		ip.setText(3, "4");
		panel.add( new JLabel("IP Address:") );
		panel.add( ip );
                System.out.println(ip.getText());
                System.out.println(ip.getData());
                System.out.println(ip.getText(1));
                System.out.println(ip.getText(2));
                System.out.println(ip.getText(3));
                System.out.println(ip.getText(4));

		//  add signed integer field
		IntegerValidator signed = new IntegerValidator(-100, 100);
		signed.setSignRequired(IntegerValidator.ALWAYS);
		TextGroup signedField = new TextGroup("+   ", "", signed);
		panel.add( new JLabel("Signed Integer:") );
		panel.add( signedField );

		//  add postal code as stand alone MaskTextField
		MaskTextField postalCode = new MaskTextField(6, "");
		postalCode.setValidator(numeric);
		panel.add( new JLabel( "Postal Code:" ) );
		panel.add( postalCode );

		//  add stand alone MaskTextField
		MaskTextField name = new MaskTextField(20, "First Name");
		name.setMaskColor( Color.blue );
		name.setDisplayPartialMask( false );
		panel.add( new JLabel("Masked First Name:") );
		panel.add( name );

		frame.pack();
		frame.setVisible(true);
	}

        public void setEnabledRecursively(boolean enable){
            setEnabledRecursively(this, enable);
        }
        private static void setEnabledRecursively(Container component, boolean enable){
            component.setEnabled(enable);
            int count = component.getComponentCount();
            for(int i = 0; i< count; i++){
                Component child = component.getComponent(i);
                if(child instanceof Container){
                    setEnabledRecursively((Container) child,enable);
                } else {
                    child.setEnabled(enable);
                }
            }
        }
}



